@@ -3,7 +3,7 @@
  * 
  * This file is a part of NSIS.
  * 
- * Copyright (C) 1999-2016 Nullsoft and Contributors
+ * Copyright (C) 1999-2017 Nullsoft and Contributors
  * 
  * Licensed under the zlib/libpng license (the "License");
  * you may not use this file except in compliance with the License.
@@ -32,7 +32,6 @@ TCHAR * NSISCALL GetNSISString(TCHAR *outbuf, int strtab);
 // use the LANG_STR_TAB() macro to decode it.
 #define GetNSISTab(strtab) (strtab < 0 ? LANG_STR_TAB(strtab) : strtab)
 
-void NSISCALL myRegGetStr(HKEY root, const TCHAR *sub, const TCHAR *name, TCHAR *out, int x64);
 #define myatoi(s) ( (int)strtoiptr(s) )
 INT_PTR NSISCALL strtoiptr(const TCHAR *s);
 #define myitoa iptrtostr
@@ -41,6 +40,27 @@ TCHAR * NSISCALL mystrcpy(TCHAR *out, const TCHAR *in);
 int NSISCALL mystrlen(const TCHAR *in);
 TCHAR * NSISCALL mystrcat(TCHAR *out, const TCHAR *concat);
 TCHAR * NSISCALL mystrstr(TCHAR *a, TCHAR *b);
+
+
+#ifndef KEY_CREATE_LINK
+#define KEY_CREATE_LINK 0x0020
+#endif
+#define KEY_FORCEVIEW KEY_CREATE_LINK // Our private flag used by RegKey* to indicate that we want it to handle HKLM[32|64] style root keys. Cannot be set if the HKEY is a real handle!
+#define KEY_ALTERVIEW SYNCHRONIZE // Our private flag used by RegKey* to indicate that we want it to apply g_exec_flags.alter_reg_view. (MSDN:"Registry keys do not support the SYNCHRONIZE standard access right")
+#define KEY_FROMSCRIPT (KEY_FORCEVIEW|KEY_ALTERVIEW) // Use this flag for registry operations from a .nsi script
+#define NSIS_REGSAM_PRIVATEMASK (KEY_FROMSCRIPT|KEY_FORCEVIEW|KEY_ALTERVIEW)
+HKEY NSISCALL GetRegKeyAndSAM(HKEY hKey, REGSAM*pRS);
+LONG NSISCALL RegKeyOpen(HKEY hBase, LPCTSTR SubKey, REGSAM RS, HKEY*phKey);
+LONG NSISCALL RegKeyCreate(HKEY hBase, LPCTSTR SubKey, REGSAM RS, HKEY*phKey);
+void NSISCALL myRegGetStr(HKEY root, const TCHAR *sub, const TCHAR *name, TCHAR *out, UINT altview);
+
+
+extern DWORD g_WinVer; // GetVersion()
+#define NSIS_WINVER_WOW64FLAG ( sizeof(void*) > 4 ? ( 0 ) : ( 0x40000000 ) )
+#define IsWow64() ( sizeof(void*) > 4 ? ( FALSE ) : ( g_WinVer & NSIS_WINVER_WOW64FLAG ) )
+#define SystemSupportsAltRegView() ( sizeof(void*) > 4 ? ( TRUE ) : ( IsWow64() ) )
+
+
 WIN32_FIND_DATA * NSISCALL file_exists(TCHAR *buf);
 TCHAR * NSISCALL my_GetTempFileName(TCHAR *buf, const TCHAR *dir);
 BOOL NSISCALL myReadFile(HANDLE h, LPVOID buf, DWORD cb);
@@ -94,6 +114,7 @@ DWORD NSISCALL CreateRestrictedDirectory(LPCTSTR path);
 DWORD NSISCALL CreateNormalDirectory(LPCTSTR path);
 
 HANDLE NSISCALL myCreateProcess(TCHAR *cmd);
+BOOL NSISCALL myShellExecuteEx(SHELLEXECUTEINFO*pSEI);
 int NSISCALL my_MessageBox(const TCHAR *text, UINT type);
 
 void NSISCALL myDelete(TCHAR *buf, int flags);
@@ -134,6 +155,9 @@ enum myGetProcAddressFunctions {
 #endif
   MGA_InitiateShutdown,
   MGA_IsUserAnAdmin,
+#ifndef _WIN64
+  MGA_IsOS,
+#endif
   MGA_SHAutoComplete, // x64 can link to shlwapi directly but as long as MGA_SHGetFolderPath is used we can stick with myGetProcAddress
   MGA_SHGetFolderPath, // TODO: This can probably call something else directly on x64
 #ifdef NSIS_SUPPORT_GETDLLVERSION
@@ -157,7 +181,9 @@ void NSISCALL MessageLoop(UINT uCheckedMsg);
  * @param funcName The name of the function to get the address of.
  * @return The pointer to the function.  Null if failure.
  */
-void * NSISCALL NSISGetProcAddress(HANDLE dllHandle, TCHAR* funcName);
+void* NSISCALL NSISGetProcAddress(HANDLE dllHandle, TCHAR* funcName);
+
+DWORD NSISCALL WaitForProcess(HANDLE hProcess);
 
 // Turn a pair of chars into a word
 // Turn four chars into a dword
