@@ -3,7 +3,7 @@
  * 
  * This file is a part of NSIS.
  * 
- * Copyright (C) 1999-2016 Nullsoft and Contributors
+ * Copyright (C) 1999-2017 Nullsoft and Contributors
  * 
  * Licensed under the zlib/libpng license (the "License");
  * you may not use this file except in compliance with the License.
@@ -32,18 +32,17 @@
 #include "winchar.h" // assert(sizeof(WINWCHAR)...)
 
 #include <nsis-version.h>
-#define NSIS_COPYYEARS _T("1999-2016")
+#define NSIS_COPYYEARS _T("1999-2017")
 
 using namespace std;
 
+NSISRT_DEFINEGLOBALS();
 bool g_dopause=false, g_warnaserror=false;
-int g_display_errors=1;
-FILE *g_output;
 NStreamEncoding g_outputenc;
 #ifdef _WIN32
 UINT g_wincon_orgoutcp;
 #ifdef _UNICODE
-WINSIO_OSDATA g_osdata_stdout;
+WINSIO_OSDATA g_osdata_stdout, g_osdata_stderr;
 #endif
 #endif
 const TCHAR *g_argv0=0;
@@ -71,7 +70,8 @@ static void myatexit()
 {
   dopause();
   ResetPrintColor();
-  if (g_output != stdout && g_output) fclose(g_output);
+  if (g_output != stdout && g_output) fclose(g_output), g_output = 0;
+  if (g_errout != stderr && g_errout) fclose(g_errout), g_errout = 0;
 #ifdef _WIN32
   SetConsoleOutputCP(g_wincon_orgoutcp);
 #endif
@@ -108,7 +108,11 @@ static DWORD WINAPI sigint_event_msg_handler(LPVOID ThreadParam)
 
   return 0;
 }
-#endif
+
+static UINT_PTR QueryHost(HWND hHost, UINT_PTR wp, UINT_PTR lp=0, UINT_PTR def=0) { return hHost ? SendMessage(hHost, MakensisAPI::QUERYHOST, wp, lp) : def; }
+#else //! _WIN32
+static inline UINT_PTR QueryHost(HWND hHost, UINT_PTR wp, UINT_PTR lp=0, UINT_PTR def=0) { return def; }
+#endif //~ _WIN32
 
 static void init_signals(HWND notify_hwnd)
 {
@@ -285,6 +289,7 @@ static inline int makensismain(int argc, TCHAR **argv)
   allow_unaligned_data_access();
 #endif
   assert(sizeof(UINT_PTR) == sizeof(void*));
+  assert('a' + 25 == 'z' && '0' < 'A' && 'A' < 'a'); // ASCII, do you speak it?
   assert(sizeof(wchar_t) > 1 && sizeof(wchar_t) <= 4);
   assert(sizeof(WINWCHAR) == 2 && sizeof(WORD) == 2);
   assert(sizeof(WINWCHAR) == sizeof(WCHAR)); // Not really required but if WCHAR changes we need to know
@@ -293,7 +298,7 @@ static inline int makensismain(int argc, TCHAR **argv)
 
   if (!NSISRT_Initialize())
   {
-    _ftprintf(stdout,_T("NSISRT_Initialize failed!\n"));
+    _ftprintf(stderr,_T("NSISRT_Initialize failed!\n"));
     return 1;
   }
 
@@ -301,11 +306,10 @@ static inline int makensismain(int argc, TCHAR **argv)
   const TCHAR*stdoutredirname=0;
   NStreamEncoding inputenc, &outputenc = g_outputenc;
   int argpos=0;
-  bool in_files=false;
-  bool do_cd=true;
+  bool do_cd=true, noconfig=false;
   bool no_logo=true;
-  bool initialparsefail=false;
-  bool noconfig=false;
+  bool initialparsefail=false, in_files=false;
+  bool oneoutputstream=false;
   signed char pponly=0;
 #ifdef _WIN32
   signed char outputbom=1;
@@ -390,17 +394,23 @@ static inline int makensismain(int argc, TCHAR **argv)
   FILE*stdoutredir=stdout;
   if (stdoutredirname) stdoutredir=my_fopen(stdoutredirname,"w");
   g_output=stdoutredir;
-  if (!g_output) g_output=stdout;
+  if (!g_output)
+    g_output=stdout; // We could not open stdoutredirname, fall back to stdout
+  else if (stdoutredirname)
+    oneoutputstream=true; // -O used, put all output in the same file
+  if (oneoutputstream || !(1 & QueryHost(hostnotifyhandle,MakensisAPI::QH_ENABLESTDERR,0,1)))
+    g_errout=g_output;
 #if defined(_WIN32) && defined(_UNICODE)
   if (hostnotifyhandle)
   {
     // The host can override the output format if they want to
     LPARAM lp=MAKELONG(outputenc.GetCodepage(),outputbom);
-    LRESULT mr=SendMessage(hostnotifyhandle,MakensisAPI::QUERYHOST,MakensisAPI::QH_OUTPUTCHARSET,lp);
+    LRESULT mr=QueryHost(hostnotifyhandle,MakensisAPI::QH_OUTPUTCHARSET,lp);
     if (mr) outputenc.SetCodepage((WORD)(--mr)), outputbom = -1;
   }
 
-  if (!WinStdIO_OStreamInit(g_osdata_stdout,g_output,outputenc.GetCodepage(),outputbom))
+  if ((                        !WinStdIO_OStreamInit(g_osdata_stdout,g_output,outputenc.GetCodepage(),outputbom))
+   || (g_errout != g_output && !WinStdIO_OStreamInit(g_osdata_stderr,g_errout,outputenc.GetCodepage(),outputbom)))
   {
     assert(!"StdIO init failed");
     return 1;
@@ -429,9 +439,9 @@ static inline int makensismain(int argc, TCHAR **argv)
 #else
   const TCHAR*const badnonwinswitchfmt=OPT_STR _T("%") NPRIs _T(" is disabled for non Win32 platforms.");
   if (hostnotifyhandle)
-    build.warning(badnonwinswitchfmt,_T("NOTIFYHWND"));
+    build.warning(DW_CMDLINE_UNSUPP_NIX,badnonwinswitchfmt,_T("NOTIFYHWND"));
   if (NStreamEncoding::UNKNOWN==outputenc.GetCodepage())
-    build.warning(badnonwinswitchfmt,_T("OUTPUTCHARSET"));
+    build.warning(DW_CMDLINE_UNSUPP_NIX,badnonwinswitchfmt,_T("OUTPUTCHARSET"));
 #endif // ~_WIN32
 
   if (!argc)
@@ -481,7 +491,7 @@ static inline int makensismain(int argc, TCHAR **argv)
         if (NStreamEncoding::UNKNOWN == cp)
         {
           if (_tcsicmp(argv[argpos], _T("AUTO")))
-            build.warning(OPT_STR _T("INPUTCHARSET: Ignoring invalid charset %") NPRIs , argv[argpos]);
+            build.warning(DW_CMDLINE_BAD_INPUTENC, OPT_STR _T("INPUTCHARSET: Ignoring invalid charset %") NPRIs , argv[argpos]);
           cp = NStreamEncoding::AUTO;
         }
         inputenc.SafeSetCodepage(cp);
@@ -512,9 +522,9 @@ static inline int makensismain(int argc, TCHAR **argv)
         };
         if (!SetPriorityClass(hProc, classes[p].priority))
           SetPriorityClass(hProc, classes[p].fallback);
-        if (p == 5) build.warning(_T("makensis is running in REALTIME priority mode!"));
+        if (p == 5) build.warning(DW_CMDLINE_HIGHPRIORITY,_T("makensis is running in REALTIME priority mode!"));
 #else
-        build.warning(badnonwinswitchfmt,_T("Px"));
+        build.warning(DW_CMDLINE_UNSUPP_NIX,badnonwinswitchfmt,_T("Px"));
 #endif
       }
       // Already parsed these (must adjust argpos)
@@ -533,7 +543,7 @@ static inline int makensismain(int argc, TCHAR **argv)
       }
       else if (S7IsChEqualI('x',swname[0]) && swname[1])
       {
-        if (build.process_oneline(swname+1,_T("<command line>"),argpos+1) != PS_OK)
+        if (build.process_oneline(swname+1,build.get_commandlinecode_filename(),argpos+1) != PS_OK)
         {
           return 1;
         }
@@ -557,7 +567,7 @@ static inline int makensismain(int argc, TCHAR **argv)
         noconfig=true;
         tstring main_conf;
         TCHAR* env_var = _tgetenv(_T("NSISCONFDIR"));
-        if(env_var == NULL)
+        if (env_var == NULL)
 #ifndef NSIS_CONFIG_CONST_DATA_PATH
           main_conf = get_dir_name(get_executable_dir(argv[0]));
 #else
@@ -694,8 +704,10 @@ int main(int argc, char **argv)
   int wargc = 0;
   wchar_t term[1], *p, **wargv = (wchar_t **) malloc((argc+1) * sizeof(void*));
   if (wargv) 
+  {
     for ( ; wargc < argc; ++wargc )
       if ((p = NSISRT_mbtowc(argv[wargc]))) wargv[wargc] = p; else break;
+  }
   if (wargc == argc)
     *term = L'\0', wargv[wargc] = term, errno = _tmain(wargc,wargv);
   else
