@@ -3,7 +3,7 @@
  * 
  * This file is a part of NSIS.
  * 
- * Copyright (C) 2011-2016 Anders Kjersem
+ * Copyright (C) 2011-2017 Anders Kjersem
  * 
  * Licensed under the zlib/libpng license (the "License");
  * you may not use this file except in compliance with the License.
@@ -32,6 +32,9 @@ bool StrSetUTF16LE(tstring&dest, const void*src)
   if (!cec.Initialize(-1,NStreamEncoding::UTF16LE)) return false;
   src = (const void*) cec.Convert(src);
   if (!src) return false;
+#endif
+#ifdef C_ASSERT
+  C_ASSERT(sizeof(tstring::value_type) >= sizeof(wchar_t));
 #endif
   try { dest = (wchar_t*) src; } catch(...) { return false; }
   return true;
@@ -328,11 +331,11 @@ WORD GetEncodingFromString(const TCHAR*s, bool&BOM)
   if (!_tcsicmp(s,_T("ACP"))) return NStreamEncoding::ACP;
   if (!_tcsicmp(s,_T("OEM"))) return NStreamEncoding::OEMCP;
   if (!_tcsicmp(s,_T("UTF8"))) return NStreamEncoding::UTF8;
-  if ((!_tcsicmp(s,_T("UTF8SIG")) || !_tcsicmp(s,_T("UTF8BOM"))) && ++BOM)
+  if ((!_tcsicmp(s,_T("UTF8SIG")) || !_tcsicmp(s,_T("UTF8BOM"))) && (BOM = true))
     return NStreamEncoding::UTF8;
-  if (!_tcsicmp(s,_T("UTF16LE")) || (!_tcsicmp(s,_T("UTF16LEBOM")) && ++BOM))
+  if (!_tcsicmp(s,_T("UTF16LE")) || (!_tcsicmp(s,_T("UTF16LEBOM")) && (BOM = true)))
     return NStreamEncoding::UTF16LE;
-  if (!_tcsicmp(s,_T("UTF16BE")) || (!_tcsicmp(s,_T("UTF16BEBOM")) && ++BOM))
+  if (!_tcsicmp(s,_T("UTF16BE")) || (!_tcsicmp(s,_T("UTF16BEBOM")) && (BOM = true)))
     return NStreamEncoding::UTF16BE;
   if (S7IsChEqualI('C',*s++) && S7IsChEqualI('P',*s++))
   {
